from enum import Enum


class GraphSource(str, Enum):
    ANSIBLE = "Ansible"
    BICEP = "Bicep"
    CLOUDFORMATION = "CloudFormation"
    DOCKERFILE = "Dockerfile"
    GITHUB_ACTIONS = "GitHubActions"
    KUBERNETES = "Kubernetes"
    TERRAFORM = "Terraform"

    def __str__(self) -> str:
        # needed, because of a Python 3.11 change
        return self.value
